// NOTE: 
// 1. PrintCE uses UNICODE strings, so this project is UNICODE too. But Win32 version of GetProcAddress() works with ASCII strings only, so there is some changes into "PrintCE.h" (see: '#define _TV(x)').
// 2. Win32 version of printce.dll supports same printers as WinCE version, but only 'FILE' port for output.
// 3. Win32 version of printce.dll supports only silent methods to setup printing settings (i.e. has no interface).
// 4. PDF output: if there is some truetype fonts into application's folder, then printce.dll will use only these fonts; in other case it will use fonts installed into OS.

#include "stdafx.h"
#include "resource.h"
#include "TCHAR.h"
#include "PrintCE.h"

HINSTANCE	hInst;
HWND		hDlg;

#define		APP_NAME	_T("Sample PrintCE for PC")

BOOL CALLBACK DialogProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void OnPrint();

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	hInst = hInstance;

    hDlg = CreateDialog(hInst, MAKEINTRESOURCE(DLG_MAIN), 0, DialogProc);

    if(!hDlg) {
        TCHAR buf [100];
        wsprintf (buf, _T("Error x%x"), GetLastError());
        MessageBox (0, buf, _T("CreateDialog"), MB_ICONEXCLAMATION | MB_OK);
        return 1;
    }

	RECT rc;
	::GetWindowRect(hDlg, &rc);
	MoveWindow(hDlg,(GetSystemMetrics(SM_CXSCREEN)-(rc.right-rc.left))/2,(GetSystemMetrics(SM_CYSCREEN)-(rc.bottom-rc.top))/2,rc.right-rc.left,rc.bottom-rc.top,FALSE);

	ShowWindow(hDlg,SW_SHOW);

    MSG  msg;
    int status;
    while((status = GetMessage (& msg, 0, 0, 0)) != 0) {
        if(status == -1)
            return -1;
        if(!IsDialogMessage(hDlg, &msg)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    return msg.wParam;
}

BOOL CALLBACK DialogProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(message) {
		case WM_INITDIALOG:
			return TRUE;
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case IDOK:
					OnPrint();
					return 0;
				case IDCANCEL:
					PostQuitMessage(0);
					return 0;
			}
			return TRUE;
		case WM_DESTROY:
			PostQuitMessage(0);
			return TRUE;
		case WM_CLOSE:
			DestroyWindow(hwnd);
			return TRUE;
	}
    return FALSE;
}

void OnPrint()
{
	double		y = 0.0;
	HBITMAP		hBmp;

	// load print library
	if(!LoadPrintLib()) {
		MessageBox(hDlg,_T("Failed to load PrintCE.DLL !"),APP_NAME,MB_OK);
		return;
	}

	// library initialization (licence key is transfered as a parameter)
	prnInit(_T("My license key"));

	// switch measuring units to inches
	prnSetMeasureUnit(kInches);

	PRN_INFO	info;

	info.printer = 9;						// Adobe PDF
	info.port = 9;							// output to file (for PC is only available port)
	info.portrait = TRUE;					// portrait mode
	info.color_mode = 2;					// CMY colors
	info.draft_mode = FALSE;				// no draft
	info.paper = 0;							// 'A4' paper for PDF printer (see "PrintCE documentation")
	info.paper_size.cx = 0;					// used only for 'custom paper'
	info.paper_size.cy = 0;					// used only for 'custom paper'
	info.start_page = info.end_page = -1;	// to print all pages
	info.user_margins.top = info.user_margins.bottom = info.user_margins.left = info.user_margins.right = 0; // zero margins

	// set output PDF path
	TCHAR	pdf_path[MAX_PATH];
	GetModuleFileName(GetModuleHandle(NULL),pdf_path,MAX_PATH);
	*_tcsrchr(pdf_path,_T('\\')) = 0;
	_tcscat(pdf_path,_T("\\sample.pdf"));
	prnSetPDFFile(pdf_path);

	// silent setup
	if(!prnSilentPrintSetup(hDlg,&info)) {
		prnUnInit();
		UnloadPrintLib();
		MessageBox(hDlg,_T("prnSilentPrintSetup failed !"),APP_NAME,MB_OK);
		return;
	}

	// set margins
	prnSetLeftMargin(1.0);
	prnSetRightMargin(1.0);
	prnSetTopMargin(0.7);
	prnSetBottomMargin(0.7);

	// start printing document
	prnStartDoc();

	// start printing page
	prnStartPage();

	// print heading with font of 14 points in height, bold 
	prnSetFontSize(14.0);
	prnSetFontBold(TRUE);
	prnDrawAlignedText(_T("PrintCE for Pocket PC"),prnGetPageWidth()/2,y,hCenter,vTop);
	y += prnGetTextHeight(_T("PrintCE for Pocket PC"));

	// print bitmap from resource
	hBmp = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_BITMAP1)); // load bitmap from resource
	if(hBmp) {
		prnDrawBitmap(hBmp,0.0,y,0.8,0.0,TRUE);
		DeleteObject(hBmp);
		y += 0.9;
	}

/*	// print picture from file
	TCHAR	pic_path[MAX_PATH];
	GetModuleFileName(GetModuleHandle(NULL),pic_path,MAX_PATH);
	*_tcsrchr(pic_path,_T('\\')) = 0;
	_tcscat(pic_path,_T("\\2.jpg"));
	prnDrawPicture(pic_path,0.0,y,0.8,0.0,TRUE);
	y += 0.9;
*/
	// set blue line colour
	prnSetLineColor(RGB(0,0,255));

	// set line width of 0.05 inches
	prnSetLineWidth(0.05);

	// print 3 blue lines of 2, 1.5 and 1 inches in length
	prnDrawLine(0,y,2.0,y);
	y += 0.3;
	prnDrawLine(0,y,1.5,y);
	y += 0.3;
	prnDrawLine(0,y,1.0,y);
	y += 0.3;
	
	// set yellow line colour
	prnSetLineColor(RGB(255,255,0));
	
	// set green fill colour
	prnSetFillColor(RGB(0,255,0));

	// print rectangle of 2 inches in length and 0.5 inch in width
	prnDrawRect(0.0, y, 2.0, y + 0.5);

	// set line width of 0.01 inches
	prnSetLineWidth(0.01);

	// set black line colour
	prnSetLineColor(RGB(0,0,0));

	// set red fill colour
	prnSetFillColor(RGB(255,0,0));

	// print ellipse inside the rectangle
	prnDrawEllipse(0.5, y + 0.05, 1.5, y + 0.45);
	y += 0.9;

	// print text with 9 points font 
	prnSetFontBold(FALSE);
	prnSetFontSize(9.0);
	prnDrawText(_T("http:\\www.inesoft.com"),0,y);
	
	// end printing document
	prnEndDoc();

	// library deinitialization
	prnUnInit();

	// Unload print library
	UnloadPrintLib();

	// show message for user
	MessageBox(hDlg,_T("Printing done !"),APP_NAME,MB_OK|MB_ICONINFORMATION);

}
