using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using PrintCENET;

namespace Sample_PrintASCII
{
    public partial class Form1 : Form
    {
        private int mCurLang = 0; // default is english language

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // 'Print' button
            Print(false);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            // 'Print Silent' button
            Print(true);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            // 'Language' button
            ContextMenu menu = new ContextMenu();
            String[] menuStr = new String[6] { "English", "Russian", "French", "Portuguese", "Italian", "Dutch" };
            for (int i = 0; i < 6; i++)
            {
                menu.MenuItems.Add(new MenuItem());
                menu.MenuItems[i].Text = menuStr[i];
                menu.MenuItems[i].Click += new EventHandler(OnLangMenuClick);
            }
            menu.MenuItems[mCurLang].Checked = true;
            menu.Show(this, new Point(this.button3.Left + this.button3.Width / 2, this.button3.Top + this.button3.Height / 2));

        }

        private void button4_Click(object sender, EventArgs e)
        {
            // 'Exit' button
            Close();
        }

        protected void OnLangMenuClick(object sender, EventArgs e)
        {
            switch (((MenuItem)sender).Text)
            {
                case "English":
                    mCurLang = 0;
                    break;
                case "Russian":
                    mCurLang = 1;
                    break;
                case "French":
                    mCurLang = 2;
                    break;
                case "Portuguese":
                    mCurLang = 3;
                    break;
                case "Italian":
                    mCurLang = 4;
                    break;
                case "Dutch":
                    mCurLang = 5;
                    break;
            }

        }

        private void Print(bool silent)
        {
            bool connected = false;

            PrintASCII print1 = null;

            try
            {
                print1 = new PrintASCII();
            }
            catch (Exception x)
            {
                MessageBox.Show(x.Message, "Error");
                return;
            }

            // output text
            String buf = "PrintASCII Test";

            // library initialization (licence key is transfered as a parameter)
            print1.Init("My license key");

            // set language of the user interface
            print1.Language = mCurLang;

            if (silent)
            {
                // don't show port setup dialog, just print to file 'silent_out.prn'
                connected = print1.ConnectSilent(ASCII_PORT.ASCII_FILE, "\\silent_out.prn");
            }
            else
            {
                // setup port
                ASCII_PORT port = (ASCII_PORT)print1.SetupPort(this.Handle);
                // connect, if selected printing to file, then set output file path
                if (port != ASCII_PORT.ASCII_NONE)
                    connected = print1.Connect(port, port == ASCII_PORT.ASCII_FILE ? "\\out.prn" : null);
            }

            if (connected)
            {
                // send string
                print1.SendString(buf);

                // send bytes
                print1.SendByte(10);
                print1.SendByte(10);
                print1.SendByte(10);

                // disconnect
                print1.Disconnect();

                // show message for user
                if (silent)
                    MessageBox.Show("Silent printing done !", "Sample PrintASCII");
            }

            // library deinitialization
            print1.UnInit();
        }
    }
}