// Sample_PDC.cpp

#include "stdafx.h"
#include "resource.h"
#include "PrintCE.h"

#define APP_NAME	_T("Sample_PDC")
#define ID_PRINT	500
#define ID_LANG		501
#define ID_EXIT		502

HINSTANCE			hInst;
HWND				hWnd;
HWND				hBtn1 = NULL;
HWND				hBtn2;
HWND				hBtn3;
int					cur_lang = 0; // english

LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL				InitInstance(HINSTANCE);
void				OnSize();
void				OnPrint();
void				OnLang();


int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
 	MSG msg;

	hInst = hInstance;

	// Perform application initialization:
	if(!InitInstance(hInstance)) 
		return FALSE;

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

BOOL InitInstance(HINSTANCE hInstance)
{
	WNDCLASS	wc;

	// register class
    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= NULL;
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= APP_NAME;

	RegisterClass(&wc);

	// create window
	hWnd = CreateWindow(APP_NAME, APP_NAME, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	if(!hWnd)
		return FALSE;

	// create buttons
	hBtn1 = CreateWindow(_T("Button"),_T("Print"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_PRINT,hInstance,NULL);
	hBtn2 = CreateWindow(_T("Button"),_T("Language"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_LANG,hInstance,NULL);
	hBtn3 = CreateWindow(_T("Button"),_T("Exit"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_EXIT,hInstance,NULL);

	if(!hBtn1 || !hBtn2)
		return FALSE;

	// resize
	OnSize();

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC			hdc;
	PAINTSTRUCT ps;

	switch(message)  {
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case ID_PRINT:
				   OnPrint();
				   break;
				case ID_LANG:
					OnLang();
					break;
				case ID_EXIT:
				   DestroyWindow(hWnd);
				   break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			OnSize();
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

void OnSize()
{
	RECT	rc;
	int		width,height;

	if(!hBtn1)
		return;

	GetClientRect(hWnd,&rc);
	width = (rc.right - rc.left)/2;
	height = (rc.bottom - rc.top)/8;

	MoveWindow(hBtn1,width/2,height,width,height,TRUE);
	MoveWindow(hBtn2,width/2,height*3,width,height,FALSE);
	MoveWindow(hBtn3,width/2,height*5,width,height,FALSE);
}

void OnLang()
{
	RECT	rc;
	HMENU	hMenu;
	int		id;

	GetClientRect(hWnd,&rc);

	hMenu = CreatePopupMenu();

	AppendMenu(hMenu,MF_STRING,1,_T("English"));
	AppendMenu(hMenu,MF_STRING,2,_T("Russian"));
	AppendMenu(hMenu,MF_STRING,3,_T("French"));
	AppendMenu(hMenu,MF_STRING,4,_T("Portuguese"));
	AppendMenu(hMenu,MF_STRING,5,_T("Italian"));
	AppendMenu(hMenu,MF_STRING,6,_T("Dutch"));

	CheckMenuItem(hMenu,cur_lang+1,MF_BYCOMMAND|MF_CHECKED);

	id = TrackPopupMenu(hMenu,TPM_BOTTOMALIGN|TPM_RIGHTALIGN|TPM_RETURNCMD,(rc.right-rc.left)/2,(rc.bottom-rc.top)/2,0,hWnd,0);

	DestroyMenu(hMenu);

	if(id == 0)
		return;

	cur_lang = id - 1;
}

void OnPrint()
{
	PRINTDLG	pdlg;
	HDC			prnDC,memDC;
	HBITMAP		hBmp, *hOldBmp;
	BITMAP		bm;
	DOCINFO		di;
	RECT		rc;

	// load print library
	if(!LoadPrintLib()) {
		MessageBox(hWnd,_T("PrintCE.dll not Found"),_T("Error"),MB_OK);
		return;
	}

	// library initialization (licence key is transfered as a parameter)
	PDC_Init(_T("My license key"));

	// set language of the user interface
	PDC_SetLanguage(cur_lang);

	// show print setup dialog
	memset(&pdlg, 0, sizeof(PRINTDLG));
	pdlg.cbStruct	= sizeof(PRINTDLG);
	pdlg.hwndOwner	=  hWnd;
	pdlg.dwFlags	= PD_INTHOUSANDTHSOFINCHES;	// measuring units - inches

	if(!PDC_PrintDlg(&pdlg)) {
		PDC_UnInit();
		UnloadPrintLib();
		return;
	}

	// get printer DC
	prnDC = PDC_GetPrinterDC();

	if(!prnDC) {
		PDC_UnInit();
		UnloadPrintLib();
		return;
	}

	// get printer dpiX, dpiY, margins (m_left,m_right,m_top,m_bottom) and page sizes (width,height) in pixels
	int		dpiX, dpiY, width, height, m_left, m_right, m_top, m_bottom;
	RECT	page_rc;

	dpiX		= PDC_GetDeviceCaps(prnDC,LOGPIXELSX);
	dpiY		= PDC_GetDeviceCaps(prnDC,LOGPIXELSY);
	m_left		= (pdlg.rcMargin.left*dpiX)/1000;
	m_right		= (pdlg.rcMargin.right*dpiX)/1000;
	m_top		= (pdlg.rcMargin.top*dpiY)/1000;
	m_bottom	= (pdlg.rcMargin.bottom*dpiY)/1000;
	width		= PDC_GetDeviceCaps(prnDC,PHYSICALWIDTH) - (m_left + m_right);
	height		= PDC_GetDeviceCaps(prnDC,PHYSICALHEIGHT) - (m_top + m_bottom);

	// page_rc is print area
	SetRect(&page_rc,m_left,m_top,m_left + width,m_top + height);

	// create fonts
	HFONT		hFont,hFontBold;
	LOGFONT		lf;

	GetObject(GetStockObject(SYSTEM_FONT),sizeof(LOGFONT),(PSTR) &lf);
	_tcscpy(lf.lfFaceName,_T("Tahoma"));

	// create "Tahoma" font of 9 points in height
	lf.lfWeight = FW_BOLD;
	lf.lfHeight = long(-(9 * PDC_GetDeviceCaps(prnDC,LOGPIXELSY) / 72.0 + 0.5));
	lf.lfWeight = FW_NORMAL;
	hFont = CreateFontIndirect(&lf);
	// create "Tahoma" font of 14 points in height, bold
	lf.lfHeight = long(-(14 * PDC_GetDeviceCaps(prnDC,LOGPIXELSY) / 72.0 + 0.5));
	lf.lfWeight = FW_BOLD;
	hFontBold = CreateFontIndirect(&lf);

	// create pens of 0.01 (black), 0.05 (yellow) and 0.05 (blue) inches in width
	HPEN	hPenBlack,hPenBlue,hPenYellow;
	hPenBlack = CreatePen(PS_SOLID,(int)(0.01 * dpiX),RGB(0,0,0));
	hPenYellow = CreatePen(PS_SOLID,(int)(0.05 * dpiX),RGB(255,255,0));
	hPenBlue = CreatePen(PS_SOLID,(int)(0.05 * dpiX),RGB(0,0,255));

	// create green and red brushes
	HBRUSH	hBrushGreen,hBrushRed;
	hBrushGreen = CreateSolidBrush(RGB(0,255,0));
	hBrushRed = CreateSolidBrush(RGB(255,0,0));

	// start printing document
	memset(&di,0,sizeof(DOCINFO));
	di.cbSize = sizeof(DOCINFO);
	PDC_StartDoc(prnDC,&di);

	// start printing page
	PDC_StartPage(prnDC);

	SetRect(&rc,page_rc.left,page_rc.top,page_rc.right,page_rc.bottom);

	// print heading
	rc.top += (int) (0.5 * dpiY);
	PDC_SelectObject(prnDC,hFontBold);
	rc.top += PDC_DrawText(prnDC,_T("PrintCE for Pocket PC"),-1,&rc,DT_TOP|DT_CENTER);

	hBmp = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_BITMAP1)); // load bitmap from resource
	if(hBmp) {
		GetObject(hBmp,sizeof(BITMAP),&bm);
		// create memory DC
		memDC = PDC_CreateCompatibleDC(prnDC);
		hOldBmp = (HBITMAP *) PDC_SelectObject(memDC,hBmp);

		// adjust bitmap size so that it occupies 0.8 inch on the page
		int bmp_width = (int) (0.8 * dpiX);
		int bmp_height = bmp_width * bm.bmHeight/bm.bmWidth * dpiY/dpiX;

		// print bitmap
		PDC_StretchBlt(prnDC,rc.left,rc.top,bmp_width,bmp_height,memDC,0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);
		rc.top += bmp_height;

		// free
		PDC_SelectObject(memDC,hOldBmp);
		PDC_DeleteDC(memDC);
	}
	rc.top += (int) (0.1 * dpiY);

	// select blue pen in print DC
	PDC_SelectObject(prnDC,hPenBlue);

	// print 3 blue lines of 2, 1.5 and 1 inches in length
	PDC_DrawLine(prnDC,rc.left,rc.top,rc.left + 2 * dpiX,rc.top);
	rc.top += (int)(0.3*dpiY);
	PDC_DrawLine(prnDC,rc.left,rc.top,rc.left + (int) (1.5 * dpiX),rc.top);
	rc.top += (int)(0.3*dpiY);
	PDC_DrawLine(prnDC,rc.left,rc.top,rc.left + 1 * dpiX,rc.top);
	rc.top += (int)(0.3*dpiY);

	// select green bruch and yellow pen in  print DC and print rectangle of 2 inches in length and 0.5 inch in width
	PDC_SelectObject(prnDC,hBrushGreen);
	PDC_SelectObject(prnDC,hPenYellow);
	PDC_Rectangle(prnDC,rc.left,rc.top,rc.left + 2 * dpiX,rc.top + (int)(0.5 * dpiY));

	// select red brush and black pen in  print DC and print ellipse inside the rectangle
	PDC_SelectObject(prnDC,hBrushRed);
	PDC_SelectObject(prnDC,hPenBlack);
	PDC_Ellipse(prnDC,rc.left + (int)(0.5 * dpiX),rc.top + (int)(0.05 * dpiX),rc.left + (int)(1.5 * dpiX),rc.top + (int)(0.45 * dpiY));

	rc.top += (int)(0.6*dpiY);

	// print text with 9 points font 
	TCHAR * adr = _T("http:\\www.inesoft.com");
	rc.top += (int)(0.3*dpiY);
	PDC_SelectObject(prnDC,hFont);
	PDC_ExtTextOut(prnDC,rc.left, rc.top, 0, NULL, adr, wcslen(adr), NULL);
	
	// end printing page
	PDC_EndPage(prnDC);

	// end printing document 
	PDC_EndDoc(prnDC);

	// clear GDI resources
	PDC_DeleteDC(prnDC);
	DeleteObject(hBmp);
	DeleteObject(hFont);
	DeleteObject(hFontBold);
	DeleteObject(hBrushRed);
	DeleteObject(hBrushGreen);
	DeleteObject(hPenBlack);
	DeleteObject(hPenYellow);
	DeleteObject(hPenBlue);

	// library deinitialization
	PDC_UnInit();

	// Unload print library
	UnloadPrintLib();
}