// Sample_Ascii.cpp

#include "stdafx.h"
#include "AsciiCE.h"

#define APP_NAME	_T("Sample_ASCII")
#define ID_PRINT	500
#define ID_SILENT	501
#define ID_LANG		502
#define ID_EXIT		503

HINSTANCE			hInst;
HWND				hWnd;
HWND				hBtn[4] = { NULL };
int					cur_lang = 0; // english

LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL				InitInstance(HINSTANCE);
void				OnSize();
void				OnPrint(BOOL silent);
void				OnLang();


int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
 	MSG msg;

	hInst = hInstance;

	// Perform application initialization:
	if(!InitInstance(hInstance)) 
		return FALSE;

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

BOOL InitInstance(HINSTANCE hInstance)
{
	WNDCLASS	wc;

	// register class
    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= NULL;
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= APP_NAME;

	RegisterClass(&wc);

	// create window
	hWnd = CreateWindow(APP_NAME, APP_NAME, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	if(!hWnd)
		return FALSE;

	// create buttons
	hBtn[0] = CreateWindow(_T("Button"),_T("Print"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_PRINT,hInstance,NULL);
	hBtn[1] = CreateWindow(_T("Button"),_T("Silent Print"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_SILENT,hInstance,NULL);
	hBtn[2] = CreateWindow(_T("Button"),_T("Language"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_LANG,hInstance,NULL);
	hBtn[3] = CreateWindow(_T("Button"),_T("Exit"),WS_CHILD|WS_VISIBLE,0,0,0,0,hWnd,(HMENU)ID_EXIT,hInstance,NULL);

	if(!hBtn[0])
		return FALSE;

	// resize
	OnSize();

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC			hdc;
	PAINTSTRUCT ps;

	switch(message)  {
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case ID_PRINT:
				   OnPrint(FALSE);
				   break;
				case ID_SILENT:
					OnPrint(TRUE);
				   break;
				case ID_LANG:
					OnLang();
					break;
				case ID_EXIT:
				   DestroyWindow(hWnd);
				   break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			OnSize();
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

void OnSize()
{
	RECT	rc;
	int		width,height,space;

	if(!hBtn[0])
		return;

	GetClientRect(hWnd,&rc);
	width = (rc.right - rc.left)/2;
	space = (rc.bottom - rc.top)/13;
	height = space * 2;

	MoveWindow(hBtn[0],width/2,space,width,height,TRUE);
	MoveWindow(hBtn[1],width/2,space*2+height,width,height,FALSE);
	MoveWindow(hBtn[2],width/2,space*3+height*2,width,height,FALSE);
	MoveWindow(hBtn[3],width/2,space*4+height*3,width,height,FALSE);
}

void OnLang()
{
	RECT	rc;
	HMENU	hMenu;
	int		id;

	GetClientRect(hWnd,&rc);

	hMenu = CreatePopupMenu();

	AppendMenu(hMenu,MF_STRING,1,_T("English"));
	AppendMenu(hMenu,MF_STRING,2,_T("Russian"));
	AppendMenu(hMenu,MF_STRING,3,_T("French"));
	AppendMenu(hMenu,MF_STRING,4,_T("Portuguese"));
	AppendMenu(hMenu,MF_STRING,5,_T("Italian"));
	AppendMenu(hMenu,MF_STRING,6,_T("Dutch"));

	CheckMenuItem(hMenu,cur_lang+1,MF_BYCOMMAND|MF_CHECKED);

	id = TrackPopupMenu(hMenu,TPM_BOTTOMALIGN|TPM_RIGHTALIGN|TPM_RETURNCMD,(rc.right-rc.left)/2,(rc.bottom-rc.top)/2,0,hWnd,0);

	DestroyMenu(hMenu);

	if(id == 0)
		return;

	cur_lang = id - 1;
}

void OnPrint(BOOL silent)
{
	TCHAR	buf[256];
	BOOL	connected = FALSE;

	// output text
	_tcscpy(buf,_T("PrintASCII Test"));

	// load print library
	if(!LoadAsciiLib()) {
		MessageBox(hWnd,_T("Can't load PrintCE.dll !"),_T("Warning"),MB_OK);
		return;
	}

	// library initialization (licence key is transfered as a parameter)
	ASCII_Init(_T("My license key"));

	// set language of the user interface
	ASCII_SetLanguage(cur_lang);

	if(silent) {
		// don't show port setup dialog, just print to file 'silent_out.prn'
		connected = ASCII_ConnectSilent(ASCII_FILE,_T("\\silent_out.prn"));
	} else {
		// setup port
		int port = ASCII_SetupPort(hWnd);
		// connect, if selected printing to file, then set output file path
		if(port != -1)
			connected = ASCII_Connect(port,port == ASCII_FILE ? _T("\\out.prn") : NULL);
	}

	if(connected) {
		// send string
		ASCII_SendString(buf);

		// send bytes
		ASCII_SendChar(10);
		ASCII_SendChar(10);
		ASCII_SendChar(10);

		// disconnect
		ASCII_Disconnect();

		// show message for user
		if(silent)
			MessageBox(hWnd,_T("Silent printing done !"),APP_NAME,MB_OK|MB_ICONINFORMATION);
	}

	// library deinitialization
	ASCII_UnInit();

	// unload print library
	UnloadAsciiLib();
}
