#ifndef __ASCIICE_H__
#define __ASCIICE_H__

#define ASCII_NONE		-1
#define ASCII_IR		0
#define ASCII_COM1		1
#define ASCII_COM2		2
#define ASCII_COM3		3
#define ASCII_COM4		4
#define ASCII_COM5		5
#define ASCII_COM6		6
#define ASCII_COM7		7
#define ASCII_COM8		8
#define ASCII_FILE		9
#define ASCII_NET		10
#define ASCII_COM9		11
#define ASCII_COM10		12
#define ASCII_COM11		13
#define ASCII_COM12		14
#define ASCII_BTBROAD	15
#define ASCII_BTMSOFT	16
#define ASCII_LPT1		17
#define ASCII_USB		18

// ASCII API
typedef void	(*ASCIIFn_Init)(LPTSTR);
typedef void	(*ASCIIFn_UnInit)();
typedef int		(*ASCIIFn_SetupPort)(HWND);
typedef int		(*ASCIIFn_SendChar)(char);
typedef int		(*ASCIIFn_SendString)(LPCTSTR);
typedef int		(*ASCIIFn_SendArray)(char *, int);
typedef int		(*ASCIIFn_Connect)(int, LPCTSTR);
typedef int		(*ASCIIFn_ConnectSilent)(int, LPCTSTR);
typedef void	(*ASCIIFn_Disconnect)();
typedef int		(*ASCIIFn_GetState)();
typedef int		(*ASCIIFn_GetSentBytes)();
typedef int		(*ASCIIFn_GetVersion)();
typedef void	(*ASCIIFn_SetComPortParam)(int, int, int, int, int, int, int, int, int, int, int, int, int);
typedef void	(*ASCIIFn_SetLanguage)(int);

ASCIIFn_Init			ASCII_Init;
ASCIIFn_UnInit			ASCII_UnInit;
ASCIIFn_SetupPort		ASCII_SetupPort;
ASCIIFn_SendChar		ASCII_SendChar;
ASCIIFn_SendString		ASCII_SendString;
ASCIIFn_SendArray		ASCII_SendArray;
ASCIIFn_Connect			ASCII_Connect;
ASCIIFn_ConnectSilent	ASCII_ConnectSilent;
ASCIIFn_Disconnect		ASCII_Disconnect;
ASCIIFn_GetState		ASCII_GetState;
ASCIIFn_GetSentBytes	ASCII_GetSentBytes;
ASCIIFn_GetVersion		ASCII_GetVersion;
ASCIIFn_SetComPortParam	ASCII_SetComPortParam;
ASCIIFn_SetLanguage		ASCII_SetLanguage;



HINSTANCE	hAsciiLib = NULL;
BOOL		gAsciiLibLoaded = FALSE;

BOOL LoadAsciiLib()
{
	if(gAsciiLibLoaded)
		return TRUE;

	hAsciiLib = LoadLibrary(_T("PrintCE.dll"));

	if(!hAsciiLib)
		return FALSE;

	ASCII_Init = (ASCIIFn_Init) ::GetProcAddress(hAsciiLib, _T("ASCII_Init"));
	if(!ASCII_Init)
		goto err_end;
	ASCII_UnInit = (ASCIIFn_UnInit) ::GetProcAddress(hAsciiLib, _T("ASCII_UnInit"));
	if(!ASCII_UnInit)
		goto err_end;
	ASCII_SetupPort = (ASCIIFn_SetupPort) ::GetProcAddress(hAsciiLib, _T("ASCII_SetupPort"));
	if(!ASCII_SetupPort)
		goto err_end;
	ASCII_SendChar = (ASCIIFn_SendChar) ::GetProcAddress(hAsciiLib, _T("ASCII_SendChar"));
	if(!ASCII_SendChar)
		goto err_end;
	ASCII_SendString = (ASCIIFn_SendString) ::GetProcAddress(hAsciiLib, _T("ASCII_SendString"));
	if(!ASCII_SendString)
		goto err_end;
	ASCII_SendArray = (ASCIIFn_SendArray) ::GetProcAddress(hAsciiLib, _T("ASCII_SendArray"));
	if(!ASCII_SendArray)
		goto err_end;
	ASCII_Connect = (ASCIIFn_Connect) ::GetProcAddress(hAsciiLib, _T("ASCII_Connect"));
	if(!ASCII_Connect)
		goto err_end;
	ASCII_ConnectSilent = (ASCIIFn_ConnectSilent) ::GetProcAddress(hAsciiLib, _T("ASCII_ConnectSilent"));
	if(!ASCII_ConnectSilent)
		goto err_end;
	ASCII_Disconnect = (ASCIIFn_Disconnect) ::GetProcAddress(hAsciiLib, _T("ASCII_Disconnect"));
	if(!ASCII_Disconnect)
		goto err_end;
	ASCII_GetState = (ASCIIFn_GetState) ::GetProcAddress(hAsciiLib, _T("ASCII_GetState"));
	if(!ASCII_GetState)
		goto err_end;
	ASCII_GetSentBytes = (ASCIIFn_GetSentBytes) ::GetProcAddress(hAsciiLib, _T("ASCII_GetSentBytes"));
	if(!ASCII_GetSentBytes)
		goto err_end;
	ASCII_GetVersion = (ASCIIFn_GetVersion) ::GetProcAddress(hAsciiLib, _T("ASCII_GetVersion"));
	if(!ASCII_GetVersion)
		goto err_end;
	ASCII_SetComPortParam = (ASCIIFn_SetComPortParam) ::GetProcAddress(hAsciiLib, _T("ASCII_SetComPortParam"));
	if(!ASCII_SetComPortParam)
		goto err_end;
	ASCII_SetLanguage = (ASCIIFn_SetLanguage) ::GetProcAddress(hAsciiLib, _T("ASCII_SetLanguage"));
	if(!ASCII_SetLanguage)
		goto err_end;

	gAsciiLibLoaded = TRUE;

	return TRUE;

err_end:

	FreeLibrary(hAsciiLib);

	hAsciiLib = NULL;

	gAsciiLibLoaded = FALSE;

	return FALSE;
}

void UnloadAsciiLib()
{
	if(!gAsciiLibLoaded)
		return;

	FreeLibrary(hAsciiLib);
	
	gAsciiLibLoaded = FALSE;
}


#endif //__ASCIICE_H__